
# q-expansion of newform 1800.4.f.j, downloaded from the LMFDB on 29 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1800
weight = 4
poly_data = [1, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 2], 1]]

hecke_ring_character_values = [[1351, [1, 0]], [901, [1, 0]], [1001, [1, 0]], [577, [-1, 0]]]
aps_data = [[0, 0], [0, 0], [0, 0], [0, 17], [-16, 0], [0, 29], [0, -35], [-4, 0], [0, 67], [-242, 0], [100, 0], [0, 219], [138, 0], [0, 89], [0, 11], [0, -81], [-268, 0], [250, 0], [0, -211], [852, 0], [0, 153], [456, 0], [0, -217], [-726, 0], [0, -689], [-126, 0], [0, -631], [0, 255], [-26, 0], [0, -621], [0, 489], [912, 0], [0, -463], [-516, 0], [-958, 0], [332, 0], [0, 511], [0, -463], [0, 327], [0, 647], [-2836, 0], [1742, 0], [-4460, 0], [0, -1891], [0, 2237], [-3608, 0], [-256, 0], [0, -2579], [0, -1113], [-2086, 0], [0, 2859], [-3624, 0], [-82, 0], [5040, 0], [0, -1155], [0, 2055], [746, 0], [-4596, 0], [0, 1103], [-8278, 0], [0, 589], [0, -53], [0, -4067], [4396, 0], [0, 2413], [0, 3513], [8808, 0], [0, -2801], [0, -3317], [-3198, 0], [0, 2615], [-312, 0], [0, -5395], [0, -2095], [6980, 0], [0, -6981], [3810, 0], [0, 4579], [-4866, 0], [-13486, 0], [5628, 0], [7938, 0], [-1916, 0], [0, -8255], [1256, 0], [0, 6111], [-5946, 0], [0, -629], [-16422, 0], [0, 1329], [0, 1843], [88, 0], [0, 7357], [7344, 0], [-1604, 0], [0, -7401], [-22514, 0], [6710, 0], [0, 3965], [4918, 0], [0, 1961], [0, 8893], [0, -10133], [13358, 0], [16360, 0], [0, 7787], [0, 3327], [0, 8871], [15840, 0], [-3002, 0], [0, 11805], [0, 11925], [0, -2667], [2164, 0], [-25220, 0], [12306, 0], [0, -13707], [0, -10917], [0, 11999], [-32004, 0], [-8526, 0], [0, 4089], [0, -8323], [0, 11223], [35336, 0], [-3482, 0], [19402, 0], [-9896, 0], [0, -247], [0, 4641], [3252, 0], [0, 2355], [25764, 0], [0, 15047], [-22362, 0], [30398, 0], [0, -645], [0, -7], [0, -19407], [27402, 0], [-28576, 0], [-31762, 0], [0, 10253], [0, 6507], [22790, 0], [23696, 0], [0, 2653], [0, -10527], [-7364, 0], [0, -8613], [0, -10601], [29490, 0], [0, 1285], [0, 18167], [0, 6237], [41132, 0], [38416, 0], [41302, 0], [0, 13075], [-35254, 0], [0, 9275], [0, -8661], [0, -17595], [40696, 0], [0, 22153], [0, 9399], [2468, 0], [0, 30543], [23854, 0], [0, -5389], [31100, 0], [-3530, 0], [-60116, 0], [0, -17735], [-49616, 0], [-7974, 0], [-5528, 0], [-9902, 0], [0, 15509], [47894, 0], [0, -3523], [39552, 0], [0, -2663], [0, 25481], [0, 23699], [-16810, 0], [0, -16069], [0, 8341], [42898, 0], [-5140, 0], [0, -4523], [0, -1401], [-9736, 0], [-46874, 0], [0, 30983], [0, 19787], [-37650, 0], [0, 4041], [0, 7729], [0, 21215], [-31698, 0], [-73604, 0], [0, -1513], [18798, 0], [86916, 0], [0, 169], [39008, 0], [0, -37325], [1290, 0], [-47752, 0], [0, -16397], [-16562, 0], [0, 10681], [0, -8337], [20560, 0], [-17562, 0], [0, -17807], [3790, 0], [0, -6249], [0, 19915], [-88894, 0], [-92800, 0], [-56750, 0], [0, -10551], [0, 35727], [25550, 0], [0, 11591], [-62656, 0], [0, 13349], [-6920, 0], [0, 15269], [-103860, 0], [-38500, 0], [27098, 0], [0, -25179], [0, 51543], [16410, 0], [0, -34185], [-85220, 0], [-15012, 0], [0, 57343], [-87536, 0], [0, -20299], [-77086, 0], [0, -5857], [54536, 0], [0, 14497], [-93848, 0], [-580, 0], [0, -33649], [0, 17787], [-42130, 0], [0, -44685], [-56702, 0], [0, 23303], [4020, 0], [-26530, 0], [0, 38925], [0, 3561], [0, -4341], [0, -47179], [0, -43697], [-99558, 0], [0, -17963], [0, -48923], [-10708, 0], [8110, 0], [-40410, 0], [0, 56769], [0, -45629], [110726, 0], [0, 18973], [0, -32043], [86704, 0], [0, -19333], [0, 5821], [0, 18707], [5890, 0], [-64234, 0], [-99552, 0], [0, -9405], [21540, 0], [0, 13867], [-35906, 0], [0, -27575], [105788, 0], [0, -55763], [0, -26571], [29040, 0], [158550, 0], [50966, 0], [0, 12927], [0, -62961], [97056, 0], [0, 52553], [-87222, 0], [13652, 0], [0, 45363], [71836, 0], [0, 83733], [0, 6081], [0, -10719], [84904, 0]]
